#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "linear.h"
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

void exit_with_help()
{
	printf(
	"Usage: train [options] training_set_file testing_set_file\n"
	"options:\n"
	"-s type : set type of solver (default 1)\n"
	"	0 -- L2-loss support vector machines (CD)\n"
	"	1 -- L2-loss support vector machines (CDPER)\n"
	"	2 -- L2-loss support vector machines (CDPERONE)\n"
	"	3 -- L2-loss support vector machines (CMLS)\n"
	"	4 -- L2-loss support vector machines (CMLSPER)\n"
	"	5 -- L2-loss support vector machines (TRON)\n"	
	"	6 -- L2 logistic regression (TRON)\n"
	"	7 -- L2 logistic regression (CDD)\n"
	"	8 -- L2 logistic regression (SCGIS)\n"
	"	9 -- L2 logistic regression (CD)\n"
	"	10 -- L2 logistic regression (CDapprox)\n"
	"-c cost : set the parameter C (default 1)\n"
	"-e epsilon : set tolerance of termination criterion\n"
	"	-s 5 and 6\n" 
	"		|f'(w)|_2 <= eps*min(pos,neg)/l*|f'(w0)|_2,\n" 
	"		where f is the primal function, (default 0.01)\n"
	"	-s 0, 1, 2\n"
	"		max(|Dp|) <=eps,\n"
	"		where Dp is the first derivative of sub-problem\n"
	"-wi weight: weights adjust the parameter C of different classes (see README for details)\n"
	"-v n: n-fold cross validation mode\n"
	);
	exit(1);
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *model_file_name);
void read_problem(const char *filename, struct problem &prob);


struct feature_node *x_space;
struct parameter param;
struct model* model_;
int flag_cross_validation;
int nr_fold;


int main(int argc, char **argv)
{
	char input_file_name[1024];
	char testing_file_name[1024];
	const char *error_msg;
	struct problem prob, prob_t;

	parse_command_line(argc, argv, input_file_name, testing_file_name);
	read_problem(input_file_name, prob);
	read_problem(testing_file_name, prob_t);
	if(prob_t.n >  prob.n)
		prob_t.n = prob.n;
	error_msg = check_parameter(&prob,&param);

	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		exit(1);
	}

	if(flag_cross_validation)
	{
		fprintf(stderr,"Not Support\n");
	}
	else
	{
		double *w = new double[prob.n];
		train_one(&prob, &prob_t, &param, w, param.C, param.C);
//		model_=train(&prob, &param);
//		save_model(model_file_name, model_);
//		destroy_model(model_);
	}
	destroy_param(&param);
	free(prob.y);
	free(prob.x);
	free(prob_t.y);
	free(prob_t.x);
	free(x_space);

	return 0;
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *testing_file_name)
{
	int i;

	// default values
	param.solver_type = L2LOSS_SVM_CDPRIMAL;
	param.C = 1;
	param.eps = INF; // see setting below
	param.nr_weight = 0;
	param.weight_label = NULL;
	param.weight = NULL;
	flag_cross_validation = 0;


	// parse options
	for(i=1;i<argc;i++)
	{
		if(argv[i][0] != '-') break;
		if(++i>=argc)
			exit_with_help();
		switch(argv[i-1][1])
		{
			case 's':
				param.solver_type = atoi(argv[i]);
				break;

			case 'c':
				param.C = atof(argv[i]);
				break;

			case 'e':
				param.eps = atof(argv[i]);
				break;


			case 'w':
				++param.nr_weight;
				param.weight_label = (int *)realloc(param.weight_label,sizeof(int)*param.nr_weight);
				param.weight = (double *)realloc(param.weight,sizeof(double)*param.nr_weight);
				param.weight_label[param.nr_weight-1] = atoi(&argv[i-1][2]);
				param.weight[param.nr_weight-1] = atof(argv[i]);
				break;

			case 'v':
				flag_cross_validation = 1;
				nr_fold = atoi(argv[i]);
				if(nr_fold < 2)
				{
					fprintf(stderr,"n-fold cross validation: n must >= 2\n");
					exit_with_help();
				}
				break;
			default:
				fprintf(stderr,"unknown option\n");
				exit_with_help();
				break;
		}
	}

	// determine filenames
	if(i>=argc)
		exit_with_help();

	strcpy(input_file_name, argv[i]);

	if(i<argc-1)
		strcpy(testing_file_name,argv[i+1]);
	else
	{
		fprintf(stderr, "For comparing testing accuracy, please assign testing file\n");
/*		char *p = strrchr(argv[i],'/');
		if(p==NULL)
			p = argv[i];
		else
			++p;
		sprintf(model_file_name,"%s.model",p);
*/
	}

	if(param.eps == INF)
	{
		if(param.solver_type == L2_LR || param.solver_type == L2LOSS_SVM || param.solver_type == L2LOSS_SVM_CDPRIMAL)
			param.eps = 0.01;

	}
}

// read in a problem (in libsvm format)
void read_problem(const char *filename, struct problem &prob)
{
	int elements, max_index, i, j;
	FILE *fp = fopen(filename,"r");

	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		exit(1);
	}

	prob.l = 0;
	elements = 0;
	while(1)
	{
		int c = fgetc(fp);
		switch(c)
		{
			case '\n':
				++prob.l;
				// fall through,
				// count the '-1' element
			case ':':
				++elements;
				break;
			case EOF:
				goto out;
			default:
				;
		}
	}
out:
	rewind(fp);

	prob.bias=-1;

	prob.y = Malloc(int,prob.l);
	prob.x = Malloc(struct feature_node *, prob.l);
	x_space = Malloc(struct feature_node, elements+prob.l);

	max_index = 0;
	j=0;
	for(i=0;i<prob.l;i++)
	{
		double label;
		prob.x[i] = &x_space[j];
		fscanf(fp,"%lf",&label);
		prob.y[i] = (int)label;

		while(1)
		{
			int c;
			do {
				c = getc(fp);
				if(c=='\n') goto out2;
			} while(isspace(c));
			ungetc(c,fp);
			if (fscanf(fp,"%d:%lf",&(x_space[j].index),&(x_space[j].value)) < 2)
			{
				fprintf(stderr,"Wrong input format at line %d\n", i+1);
				exit(1);
			}
			if (x_space[j].index<=0)
			{
				fprintf(stderr,"Error: index <=0\n");
				exit(1);
			}
			++j;
		}
out2:
		if(j>=1 && x_space[j-1].index > max_index)
			max_index = x_space[j-1].index;

		if(prob.bias>=0)
			x_space[j++].value = prob.bias;

		x_space[j++].index = -1;
	}

	prob.n=max_index;

	fclose(fp);
}
